@@ -271,7 +271,7 @@ attr2mode(uint16_t attr)
  * NULL terminated (but with unknown contents). 
  *
  */
-TSK_RETVAL_ENUM
+static TSK_RETVAL_ENUM
 fatfs_dinode_copy(FATFS_INFO * fatfs, TSK_FS_META * fs_meta,
     fatfs_dentry * in, TSK_DADDR_T sect, TSK_INUM_T inum)
 {
@@ -473,6 +473,16 @@ fatfs_dinode_copy(FATFS_INFO * fatfs, TSK_FS_META * fs_meta,
                 fs_meta->name2->name[i++] = in->ext[a];
         }
         fs_meta->name2->name[i] = '\0';
+
+        /* clean up non-ASCII because we are
+         * copying it into a buffer that is supposed to be UTF-8 and
+         * we don't know what encoding it is actually in or if it is 
+         * simply junk. */
+        for (i = 0; fs_meta->name2->name[i] != '\0'; i++) {
+            if ((unsigned char)(fs_meta->name2->name[i]) > 0x7e) {
+                fs_meta->name2->name[i] = '^';
+            }
+        }
     }
     /* If the entry is a normal short entry, then copy the name
      * and add the '.' for the extension
@@ -503,6 +513,16 @@ fatfs_dinode_copy(FATFS_INFO * fatfs, TSK_FS_META * fs_meta,
             }
         }
         fs_meta->name2->name[i] = '\0';
+
+        /* clean up non-ASCII because we are
+         * copying it into a buffer that is supposed to be UTF-8 and
+         * we don't know what encoding it is actually in or if it is 
+         * simply junk. */
+        for (i = 0; fs_meta->name2->name[i] != '\0'; i++) {
+            if ((unsigned char)(fs_meta->name2->name[i]) > 0x7e) {
+                fs_meta->name2->name[i] = '^';
+            }
+        }
     }
 
     /* Clean up name to remove control characters */
@@ -994,6 +1014,7 @@ fatfs_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start_inum,
     uint8_t *sect_alloc;
     ssize_t cnt;
     uint8_t done = 0;
+    char *dinode_buf = NULL;
 
     // clean up any error messages that are lying around
     tsk_error_reset();
@@ -1193,6 +1214,12 @@ fatfs_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start_inum,
         return 1;
     }
 
+    if ((dinode_buf = tsk_malloc(fatfs->csize * fatfs->ssize)) == NULL) {
+        tsk_fs_file_close(fs_file);
+        free(sect_alloc);
+        return 1;
+    }
+
     sect = ssect;
     while (sect <= lsect) {
         int clustalloc;         // 1 if current sector / cluster is allocated
@@ -1216,7 +1243,7 @@ fatfs_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start_inum,
 
             /* read the sector */
             cnt =
-                tsk_fs_read_block(fs, sect, fatfs->dinodes, fatfs->ssize);
+                tsk_fs_read_block(fs, sect, dinode_buf, fatfs->ssize);
             if (cnt != fatfs->ssize) {
                 if (cnt >= 0) {
                     tsk_error_reset();
@@ -1227,6 +1254,7 @@ fatfs_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start_inum,
                     sect);
                 tsk_fs_file_close(fs_file);
                 free(sect_alloc);
+                free(dinode_buf);
                 return 1;
             }
             sect_proc = 1;
@@ -1248,6 +1276,7 @@ fatfs_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start_inum,
             if (clustalloc == -1) {
                 tsk_fs_file_close(fs_file);
                 free(sect_alloc);
+                free(dinode_buf);
                 return 1;
             }
             else if ((clustalloc == 0)
@@ -1274,7 +1303,7 @@ fatfs_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start_inum,
 
             /* read the full cluster */
             cnt = tsk_fs_read_block
-                (fs, sect, fatfs->dinodes, sect_proc << fatfs->ssize_sh);
+                (fs, sect, dinode_buf, sect_proc << fatfs->ssize_sh);
             if (cnt != (sect_proc << fatfs->ssize_sh)) {
                 if (cnt >= 0) {
                     tsk_error_reset();
@@ -1284,6 +1313,7 @@ fatfs_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start_inum,
                     "fatfs_inode_walk: sector: %" PRIuDADDR, sect);
                 tsk_fs_file_close(fs_file);
                 free(sect_alloc);
+                free(dinode_buf);
                 return 1;
             }
         }
@@ -1294,7 +1324,7 @@ fatfs_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start_inum,
             uint8_t isInDir;
 
             dep =
-                (fatfs_dentry *) & fatfs->dinodes[sidx << fatfs->ssize_sh];
+                (fatfs_dentry *) & dinode_buf[sidx << fatfs->ssize_sh];
 
             /* if we know it is not part of a directory and it is not valid dentires,
              * then skip it */
@@ -1401,6 +1431,7 @@ fatfs_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start_inum,
                     else {
                         tsk_fs_file_close(fs_file);
                         free(sect_alloc);
+                        free(dinode_buf);
                         return 1;
                     }
                 }
@@ -1415,11 +1446,13 @@ fatfs_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start_inum,
                 if (retval == TSK_WALK_STOP) {
                     tsk_fs_file_close(fs_file);
                     free(sect_alloc);
+                    free(dinode_buf);
                     return 0;
                 }
                 else if (retval == TSK_WALK_ERROR) {
                     tsk_fs_file_close(fs_file);
                     free(sect_alloc);
+                    free(dinode_buf);
                     return 1;
                 }
             }                   /* dentries */
@@ -1433,6 +1466,8 @@ fatfs_inode_walk(TSK_FS_INFO * fs, TSK_INUM_T start_inum,
 
 
     free(sect_alloc);
+    free(dinode_buf);
+    dinode_buf = NULL;
 
 
     // handle the virtual orphans folder and FAT files if they asked for them
