@@ -38,7 +38,7 @@
 ssize_t
 tsk_fs_read(TSK_FS_INFO * a_fs, TSK_OFF_T a_off, char *a_buf, size_t a_len)
 {
-    TSK_OFF_T off;
+    TSK_OFF_T off = 0;
 
     // do a sanity check on the read bounds, but only if the block
     // value has been set. 
@@ -60,8 +60,41 @@ tsk_fs_read(TSK_FS_INFO * a_fs, TSK_OFF_T a_off, char *a_buf, size_t a_len)
         return -1;
     }
 
-    off = a_off + a_fs->offset;
-    return tsk_img_read(a_fs->img_info, off, a_buf, a_len);
+    off = a_off + a_fs->offset; 
+    
+    if (((a_fs->block_pre_size) || (a_fs->block_post_size)) && (a_fs->block_size)) {
+        TSK_OFF_T cur_off = a_off;
+        ssize_t retval = 0;
+        TSK_OFF_T end_addr = a_off + a_len;
+
+        // we need to read block by block so that we can skip the needed pre and post bytes
+        while (cur_off < end_addr) {
+            TSK_DADDR_T blk = cur_off / a_fs->block_size;
+            TSK_OFF_T read_off = off;
+            ssize_t retval2 = 0;
+            size_t read_len = a_fs->block_size - cur_off % a_fs->block_size;
+            
+            if (read_len + cur_off > end_addr) 
+                read_len = end_addr - cur_off;
+                        
+            if (a_fs->block_pre_size)
+                read_off += ((blk+1) * a_fs->block_pre_size);
+            if (a_fs->block_post_size) 
+                read_off += (blk * a_fs->block_post_size);
+            
+            retval2 = tsk_img_read(a_fs->img_info, read_off, &a_buf[retval], read_len);
+            if (retval2 == -1) 
+                return -1;
+            else if (retval2 == 0)
+                break;
+            retval += retval2;
+            cur_off += retval2;
+        }
+        return retval;
+    }
+    else {
+        return tsk_img_read(a_fs->img_info, off, a_buf, a_len);
+    }
 }
 
 
@@ -82,6 +115,8 @@ ssize_t
 tsk_fs_read_block(TSK_FS_INFO * a_fs, TSK_DADDR_T a_addr, char *a_buf,
     size_t a_len)
 {
+    TSK_OFF_T off = 0;
+
     if (a_len % a_fs->block_size) {
         tsk_error_reset();
         tsk_errno = TSK_ERR_FS_READ;
@@ -104,8 +139,31 @@ tsk_fs_read_block(TSK_FS_INFO * a_fs, TSK_DADDR_T a_addr, char *a_buf,
                 PRIuDADDR ")", a_addr);
         return -1;
     }
+    
+    off = a_fs->offset + (TSK_OFF_T) (a_addr) * a_fs->block_size;
+    if ((a_fs->block_pre_size == 0) && (a_fs->block_post_size == 0)) {
+        return tsk_img_read(a_fs->img_info, off, a_buf, a_len);
+    }
+    else {
+        size_t i;
+        ssize_t retval = 0;
+
+        for (i = 0; i < a_len; i++) {
+            ssize_t retval2;
+            TSK_OFF_T off2 = off + i*a_fs->block_size;
+            off += ((a_addr+1) * a_fs->block_pre_size);
+            off += (a_addr * a_fs->block_post_size);
+            
+            retval2 = tsk_img_read(a_fs->img_info, off2, &a_buf[retval], a_fs->block_size);
+            if (retval2 == -1)
+                return -1;
+            else if (retval2 == 0)
+                break;
+
+            retval += retval2;
+        }
+        return retval;
+    }
+
 
-    return tsk_img_read(a_fs->img_info,
-        a_fs->offset + (TSK_OFF_T) a_addr * a_fs->block_size, a_buf,
-        a_len);
 }
